#include <HT32F125x.h>                    // 


#include <stdio.h>

volatile unsigned long SysTickCnt;      // SysTick Counter

// SysTick Setup
void SysTick_Setup (void)
{
  SysTick->LOAD = 72000 - 1;            // 1ms: 72000 ticks @ 72MHz
  SysTick->CTRL = 0x0007;               // Enable Counter and Tick Interrupt
}

// SysTick Interrupt Handler (tick each 1ms)
void SysTick_Handler (void)
{
  SysTickCnt++;
}


// Delay Function
void Delay (unsigned long tick)
{
unsigned long systickcnt;

  systickcnt = SysTickCnt;
  while ((SysTickCnt - systickcnt) < tick);
}


void init_ADC(void)
{
	HT_CKCU->APBCCR0 |= (1<<14);		// 14=AFIOEN, enable APB clocks for AFIO
	HT_CKCU->APBCCR1 |= (1<<24);		// 24=ADCEN, enable APB clocks for ADC
/* PA0 input circuit disabled by default, otherwise must enable GPIO APB clock in CKCU->APBCCR0 
	HT_GPIOA->INER &= ~(1<<0);		// PA0 input circuit disabled
*/
	HT_AFIO->GPACFGR = 0x01;		// PA0 -> AF1 (ADC)
	HT_CKCU->APBCFGR= (0x06<<16);		// set ADC clock to APB/64 (72/64 = 1,125Mhz)
	// Sampling time = (ADSTn [7:0] + 1.5) A/D Converter clock cycles.
	HT_ADC->STR[0] = 10;			// 11.5 clock cycles -> ~10us
	// List of channels to convert, in sequence
	HT_ADC->LST[0] = 0 + (0<<8) + (0<<16) + (0<<24); // PA0
	// Channels to convert -1 , conversion mode: 0 -> single, 2 -> continuous
	HT_ADC->CONV = (0<<8) + 0;		// 1 channel, single
	// Start conversion source: b0=ADSW, b1=ADEXTI, b2=ADTM
	HT_ADC->TCR = (1<<0);			// software
//	HT_ADC->TSR = ;				// set EXTI and GPTM sources

#ifdef ADC_USEINT
	NVIC_EnableIRQ(ADC_IRQn);		// Enable ADC Interrupts
	HT_ADC->IM |= (1<<2);			// enable ADIMC
#endif
}

// reset ADIRAWC and set ADSC to start conversion
void ADC_start(void)
{
#ifndef ADC_USEINT
	HT_ADC->ICLR=(1<<2);
#endif
	HT_ADC->TSR |= (1<<0);
}

// read ADIRAWC
#define ADC_ready()	(HT_ADC->IRAW&(1<<2))
// read DR[ch]
#define ADC_read(ch)	((uint16_t)HT_ADC->DR[ch])
// test DR[ch]
#define ADC_valid(ch)	(HT_ADC->DR[ch]&(1U<<31))

uint16_t ADC_waitvalid(unsigned int ch)
{
uint32_t val;

	while(!((val=HT_ADC->DR[ch])&(1U<<31)));
	return((uint16_t)val);
}
		


#ifdef ADC_USEINT
void ADC_IRQHandler(void)
{
static uint16_t d;

	HT_ADC->ICLR = (1<<2);			// reset ADIRAWC
	d=ADC_read(0);				// get conversion result
}
#endif

/* MAIN PROGRAM */

main()
{
#ifndef ADC_USEINT
static int i;
#endif

	SysTick_Setup();
	init_ADC();
	ADC_start();				// start conversion
	while(1){
#ifndef ADC_USEINT
		while(!ADC_ready());
#if 0		// either wait a reasonable time (some ADC clocks)
		Delay(2);
		i=ADC_read(0);			// get conversion result
#endif		// or busy wait (loop) until flag gets set
		i=ADC_waitvalid(0);
#endif
		Delay(1000);
		ADC_start();			// start conversion
	}
}

